#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse

from obspy.geodetics.base import gps2dist_azimuth


def main(infile):
    per = 4e4 # earth perimeter

    fid = open(infile, 'r')
    nrec = fid.readline()
    latsrc, lonsrc, depsrc = [float(value) for value in fid.readline().split()]
    phase = fid.readline()
    latrec, lonrec, deprec = [float(value) for value in fid.readline().split()[:3]]

    print ('Lat/Lon Source: {} / {}'.format(latsrc, lonsrc))
    print ('Lat/Lon Receiver: {} / {}'.format(latrec, lonrec))

    dist_deg = gps2dist_azimuth(latsrc, lonsrc, latrec, lonrec)[0] / 1e3 / per * 360
    print ('Distance: {} [deg]'.format(dist_deg))

if __name__ == "__main__":
    parser = argparse.ArgumentParser('Estimate distance from source to first'
                                     ' receiver in WGS84 ellipsoid in FMTOMO pick file.')
    parser.add_argument('infile', help='FMTOMO pickfile (*.ttf)')
    args = parser.parse_args()
    main(args.infile)
