#!/usr/bin/env python
# -*- coding: utf-8 -*-

import numpy as np
import json

from pylot.tomography.fmtomo_tools.fmtomo_grid_utils import read_vgrid, write_vtk, calculate_differences_grid, write_vgrid

def visualize_frechet_derivative(fname_vgrid, fname_frechet, fname_out_vtk=None, fname_out_json=None,
                                 fname_out_vgrid=None, diff_model=None, fname_vgrid_ref=None):
    vgrid, gridN, gridDelta, gridStart = read_vgrid(fname_vgrid, inv_index_frechet=True)
    if diff_model and fname_vgrid_ref:
        raise OverflowError('Cannot have both parameters set, diff_model and fname_vgrid_ref')
    if diff_model:
        vgrid = calculate_differences_grid(vgrid, earth_model=diff_model)
    if fname_vgrid_ref:
        vgrid_ref, gridN_ref, gridDelta_ref, gridStart_ref = read_vgrid(fname_vgrid_ref, inv_index_frechet=False)
        grid_check = (gridN == gridN_ref,
                      compare_tuple(gridDelta, gridDelta_ref),
                      compare_tuple(gridStart, gridStart_ref))
        assert(all(grid_check), 'Missmatch ref grid size')
        vps = np.array(vgrid['vps'])
        vps_ref = np.array(vgrid_ref['vps'])
        vps_rel = (vps - vps_ref) / vps_ref * 100.
        print('Min/Max change {}/{}%'.format(min(vps_rel), max(vps_rel)))
        vgrid['vps'] = list(vps_rel)

    add_frechet(vgrid, fname_frechet)

    if fname_out_vgrid:
        write_vgrid(vgrid, gridN, gridDelta, gridStart, fname_out_vgrid)
    if fname_out_vtk:
        write_vtk(vgrid, fname_out_vtk, ['vps', 'frechs', 'grid_indices', 'hit_count'])
    if fname_out_json:
        with open(fname_out_vtk, 'w') as outfile:
            json.dump(vgrid, outfile)


def add_frechet(vgrid, fname_frechet):
    vgrid['frechs'] = list(np.zeros(len(vgrid['xs'])))
    vgrid['hit_count'] = list(np.zeros(len(vgrid['xs'])))
    with open(fname_frechet, 'r') as infile:
        while True:
            try:
                n, source_id, m, k, n_pdev = [int(item) for item in infile.readline().split()]
            except:
                break
            #print(n, source_ids, m, k, n_pdev)
            for _ in range(n_pdev):
                pdev_index, pdev = infile.readline().split()
                pdev_index = int(pdev_index)
                pdev = float(pdev)
                vgrid_index = vgrid['inv_index'][pdev_index]
                vgrid['frechs'][vgrid_index] += pdev
                # hit by ray count
                vgrid['hit_count'][vgrid_index] += 1


def compare_tuple(t1, t2, epsilon=1e-6):
    for item1, item2 in zip(t1, t2):
        if abs(item1 - item2) > epsilon:
            return False
    return True
